// Written by Jason Heddings: http://syncmarks.mozdev.org/
// License: http://creativecommons.org/licenses/by-nc-sa/2.0/

function MemotooFirefoxBookmarks(MemotooFirefox) { this._init(MemotooFirefox); }

MemotooFirefoxBookmarks.prototype = 
{
  _MemotooFirefox:null,

  _rootFolder:null,

////////////////////////////////////////////////////////////////////////////////
// Initialize the bookmarks iterator.

  _init:function(MemotooFirefox) 
  {
    this._MemotooFirefox = MemotooFirefox;

    this._rootFolder = RDF.GetResource("NC:BookmarksRoot");
  },

////////////////////////////////////////////////////////////////////////////////
// Returns the bookmark root specified by the user's settings.

  get bookmarks() 
  {
    BMSVC.readBookmarks();
    return this._getTree(RDF.GetResource(this._MemotooFirefox.settings.prefDossier));
  },

////////////////////////////////////////////////////////////////////////////////
// Returns all sub-nodes of a specified starting folder as a tree.

  _getTree:function(res) 
  { 	
    var container = Components.classes["@mozilla.org/rdf/container;1"]
                    .createInstance(Components.interfaces.nsIRDFContainer);
    container.Init(BMDS, res);

    var bookmarks = new Array();
    var elements = container.GetElements();

    // loop over all the elements in the folder
    while (elements.hasMoreElements()) {

      // add the current element to our array
      var element = elements.getNext().QueryInterface(Components.interfaces.nsIRDFResource);
      var bmark = new MemotooFirefoxBookmark(element, this._MemotooFirefox);
      bookmarks.push(bmark);

      switch (bmark.type) {
        // if the bookmark is a folder, set its children...
        case "Folder":
        case "PersonalToolbarFolder":
        case "IEFavoriteFolder":
          bmark.children = this._getTree(element);
          break;

        // if the bookmark is a regular bookmark, don't do anything special
        case "BookmarkSeparator":
        case "IEFavorite":
        case "ImmutableBookmark":
        case "Bookmark":
        case "Livemark":
        default: break;
      }
    }

    return bookmarks;
  },

};

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

function MemotooFirefoxBookmark(res, MemotooFirefox) 
{
  this._init(res, MemotooFirefox);
}

MemotooFirefoxBookmark.prototype = 
{
  _res:null,
  _MemotooFirefox:null,
  _bookmarks:null,

  children:null,

////////////////////////////////////////////////////////////////////////////////

  _init:function(res, MemotooFirefox) 
  {
    this._res = res;
    this._MemotooFirefox = MemotooFirefox;
    this._bookmarks = this._MemotooFirefox.bookmarks;
  },

////////////////////////////////////////////////////////////////////////////////

  get id()        { return escape(this.uri); },
  get uri()       { return this._res.Value; },
  get url()       { return this._getLiteral(this._res, this._urlArc); },
  get name()      { return this._getLiteral(this._res, this._nameArc); },
  get desc()      { return this._getLiteral(this._res, this._descArc); },
  get schedule()  { return this._getLiteral(this._res, this._dateSchedArc); },
  get addDate()   { return this._getDate(this._res, this._dateAddArc); },
  get modDate()   { return this._getDate(this._res, this._dateModArc); },
  get visitDate() { return this._getDate(this._res, this._dateVisitArc); },

////////////////////////////////////////////////////////////////////////////////

  // Known types:
  //  o BookmarkSeparator
  //  o IEFavorite
  //  o ImmutableBookmark
  //  o Bookmark
  //  o Livemark
  //  o Folder
  //  o PersonalToolbarFolder
  //  o IEFavoriteFolder

  get type() 
  {
    var type = BMDS.GetTarget(this._res, this._typeArc, true);
    if (this._res.Value == "NC:PersonalToolbarFolder") {
      return "PersonalToolbarFolder";
    } else if (type) {
      var uri = type.QueryInterface(Components.interfaces.nsIRDFResource).Value;
      return uri.split("#")[1];
    } else if (this._rdfcu.IsContainer(BMDS, this._res)) {
      return "Folder";
    } else if (this._getLiteral(this._res, this._urlArc)) {
      return "Bookmark";
    }
    return "";
  },

////////////////////////////////////////////////////////////////////////////////

  get _typeArc()      { return RDF.GetResource("http://www.w3.org/1999/02/22-rdf-syntax-ns#type"); },
  get _idArc()        { return RDF.GetResource("http://home.netscape.com/NC-rdf#ID"); },
  get _urlArc()       { return RDF.GetResource("http://home.netscape.com/NC-rdf#URL"); },
  get _nameArc()      { return RDF.GetResource("http://home.netscape.com/NC-rdf#Name"); },
  get _descArc()      { return RDF.GetResource("http://home.netscape.com/NC-rdf#Description"); },
  get _keywordArc()   { return RDF.GetResource("http://home.netscape.com/NC-rdf#ShortcutURL"); },
  get _iconArc()      { return RDF.GetResource("http://home.netscape.com/NC-rdf#Icon"); },
  get _feedArc()      { return RDF.GetResource("http://home.netscape.com/NC-rdf#FeedURL"); },
  get _sideArc()      { return RDF.GetResource("http://home.netscape.com/NC-rdf#WebPanel"); },
  get _dateAddArc()   { return RDF.GetResource("http://home.netscape.com/NC-rdf#BookmarkAddDate"); },
  get _dateModArc()   { return RDF.GetResource("http://home.netscape.com/WEB-rdf#LastModifiedDate"); },
  get _dateVisitArc() { return RDF.GetResource("http://home.netscape.com/WEB-rdf#LastVisitDate"); },
  get _dateSchedArc() { return RDF.GetResource("http://home.netscape.com/WEB-rdf#Schedule"); },

////////////////////////////////////////////////////////////////////////////////

  _getLiteral:function(res, arc) 
  {
    try {
      var target = BMDS.GetTarget(res, arc, true);          
      if (target) {
        return target.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
      }
    } catch (e) { /* probably a bad interface */ }
    return null;
  },

////////////////////////////////////////////////////////////////////////////////

  _getDate:function(res, arc) 
  {
    try {
      var target = BMDS.GetTarget(res, arc, true);          
      if (target) {
        return target.QueryInterface(Components.interfaces.nsIRDFDate).Value / 1000;
      }
    } catch (e) { /* probably a bad interface */ }
    return null;
  },

};
